package org.nakedobjects.nos.remote.command.marshal;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.nof.reflect.remote.data.Data;
import org.nakedobjects.nof.reflect.remote.data.DummyNullValue;
import org.nakedobjects.nof.reflect.remote.data.DummyObjectData;
import org.nakedobjects.nof.reflect.remote.data.DummyValueData;
import org.nakedobjects.nof.reflect.remote.data.NullData;
import org.nakedobjects.testing.DummyNakedObject;
import org.nakedobjects.testing.DummyOid;
import org.nakedobjects.testing.DummyVersion;
import org.nakedobjects.testing.TestSystem;

import test.org.nakedobjects.objects.bom.Movie;
import test.org.nakedobjects.objects.bom.Person;
import test.org.nakedobjects.objects.specification.MovieSpecification;

public class MarshallerTest extends TestCase {

    private Marshaller encoder;
    private TestSystem system;

    protected void setUp() throws Exception {
        BasicConfigurator.configure();
        LogManager.getRootLogger().setLevel(Level.OFF);

        system = new TestSystem();
        system.addSpecificationToLoader(new MovieSpecification());
    //    system.addSpecification(new PersonSpecification());
    //    system.addSpecification(new TestSpecification(String.class));
        system.init();

        encoder = new Marshaller();
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }
    
    public void testRestoreCreatesNewAdapter() {
        DummyObjectData data = new DummyObjectData(new DummyOid(5, true), Movie.class.getName(), true, new DummyVersion(3));
        DummyValueData name = new DummyValueData("ET", "java.lang.String");
        DummyNullValue reference = new DummyNullValue(Person.class.getName());
        // note - the order of the fields is by name, not the order that are defined in the specification
        data.setFieldContent(new Data[] { reference, name });

        NakedObject object = (NakedObject) encoder.restore(data);
        
        assertTrue(object.getObject() instanceof Movie);
        
        Movie movie = (Movie) object.getObject();
        assertEquals("ET", movie.getName());
        assertEquals(new DummyOid(5, true), object.getOid());
        assertEquals(new DummyVersion(3), object.getVersion());
    }

    
    public void testRestoreCreatesNewAdapterInUnresolvedState() {
        DummyObjectData data = new DummyObjectData(new DummyOid(5, true), Movie.class.getName(), false, new DummyVersion(3));

        NakedObject object = (NakedObject) encoder.restore(data);
        
        assertTrue(object.getObject() instanceof Movie);
        
        assertEquals(new DummyOid(5, true), object.getOid());
        assertEquals(null, object.getVersion());
    }

    public void testRestoreUpdatesExistingAdapter() {
        final Movie movie = new Movie();
        DummyNakedObject adapter = new DummyNakedObject(movie);
        adapter.setupResolveState(ResolveState.RESOLVED);
        adapter.setupSpecification(new MovieSpecification());
        system.addIdentityToObjectLoader(new DummyOid(4, true), adapter);
       
        DummyObjectData data = new DummyObjectData(new DummyOid(4, true), Movie.class.getName(), true, new DummyVersion(3));
        DummyValueData name = new DummyValueData("ET", "java.lang.String");
        DummyNullValue reference = new DummyNullValue(Person.class.getName());
        data.setFieldContent(new Data[] { reference, name });

        // after restore updated object is added to dirty object set
        /*
        MockControl control = MockControl.createControl(DirtyObjectSet.class);
        DirtyObjectSet updateNotifier = (DirtyObjectSet) control.getMock();
        encoder.setUpdateNotifer(updateNotifier);
        updateNotifier.addDirty(adapter);
        control.replay();
        */
        NakedObject object = (NakedObject) encoder.restore(data);
        
        //control.verify();
        
        assertEquals(movie, object.getObject());
        assertEquals(new DummyVersion(3), object.getVersion());
        assertEquals("ET", movie.getName());
    }
    

    public void testRestoreIgnoredIfNoFieldData() {
        final Movie movie = new Movie();
        DummyNakedObject adapter = new DummyNakedObject(movie);
        adapter.setupResolveState(ResolveState.GHOST);
        adapter.setupSpecification(new MovieSpecification());
        adapter.setupVersion(new DummyVersion(1));
        system.addIdentityToObjectLoader(new DummyOid(4, true), adapter);
       
        DummyObjectData data = new DummyObjectData(new DummyOid(4, true), Movie.class.getName(), false, new DummyVersion(3));

        NakedObject object = (NakedObject) encoder.restore(data);
        
        assertEquals(movie, object.getObject());
        assertEquals(new DummyVersion(1), object.getVersion());
    }
    
    public void testRestoreTransientObject() {
        DummyObjectData movieData = new DummyObjectData(new DummyOid(-1), Movie.class.getName(), true, null);
        NullData directorData = new DummyNullValue(Person.class.getName());
        DummyValueData nameData = new DummyValueData("Star Wars", String.class.getName());
        movieData.setFieldContent(new Data[] { directorData, nameData });
  
        NakedObject object = (NakedObject) encoder.restore(movieData);
        Movie movie = (Movie) object.getObject();
        
        assertEquals(movie, object.getObject());
        assertEquals(new DummyOid(-1), object.getOid());
        assertEquals(ResolveState.TRANSIENT, object.getResolveState());
        assertEquals(null, object.getVersion());

    }
}


// Copyright (c) Naked Objects Group Ltd.
