package org.nakedobjects.nos.remote.command.marshal;

import java.util.Enumeration;
import java.util.Vector;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.nof.reflect.remote.data.CollectionData;
import org.nakedobjects.nof.reflect.remote.data.DummyCollectionData;
import org.nakedobjects.nof.reflect.remote.data.DummyObjectData;
import org.nakedobjects.nof.reflect.remote.data.ObjectData;
import org.nakedobjects.nof.reflect.remote.data.ReferenceData;
import org.nakedobjects.testing.DummyOid;
import org.nakedobjects.testing.DummyVersion;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;

import test.org.nakedobjects.object.persistence.defaults.TestObject;


public class ObjectDecoderCollectionTest extends TestCase {

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(ObjectDecoderCollectionTest.class);
    }

    private TestSystem system;
    private BasicObjectDeserializer deserializer;

    protected void setUp() throws Exception {
        LogManager.getRootLogger().setLevel(Level.OFF);
        system = new TestSystem();

        system.addSpecificationToLoader(new TestSpecification("type.1") {
            public Object newInstance() {
                return new TestObject();
            }
        });
        TestSpecification collectionType = new TestSpecification("type.2");
        collectionType.setupIsCollection();
        system.addSpecificationToLoader(collectionType);

        system.init();
        
        deserializer = new BasicObjectDeserializer();
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }

    public void testRecreateCollection() {
        ReferenceData elements[] = new ObjectData[2];
        DummyOid fieldOid = new DummyOid(345, true);
        elements[0] = new DummyObjectData(fieldOid, "type.1", false, new DummyVersion(3));
        fieldOid = new DummyOid(678, true);
        elements[1] = new DummyObjectData(fieldOid, "type.1", false, new DummyVersion(7));

        DummyOid collectionOid = new DummyOid(123);
        CollectionData data = new DummyCollectionData(collectionOid, "type.2", "type.1", elements, new DummyVersion());

        NakedCollection naked = (NakedCollection) deserializer.restore(data);

        Vector restoredCollection = (Vector) naked.getObject();
        assertEquals(2, restoredCollection.size());

        Enumeration elements2 = naked.elements();
        NakedObject element0 = (NakedObject) elements2.nextElement();
        NakedObject element2 = (NakedObject) elements2.nextElement();

        assertNotNull(element0.getObject());
        assertNotNull(element2.getObject());
        
        assertEquals(TestObject.class, element0.getObject().getClass());
        assertEquals(TestObject.class, element2.getObject().getClass());

        assertEquals(new DummyOid(345, true), element0.getOid());
        assertEquals(new DummyOid(678, true), element2.getOid());
        
        // version not set as there is no field data for elements
       // assertEquals(new DummyVersion(3), naked.elementAt(0).getVersion());
       // assertEquals(new DummyVersion(7), naked.elementAt(1).getVersion());
    }
}
// Copyright (c) Naked Objects Group Ltd.
