package org.nakedobjects.nos.remote.command.marshal;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.nof.reflect.remote.data.CollectionData;
import org.nakedobjects.nof.reflect.remote.data.Data;
import org.nakedobjects.nof.reflect.remote.data.DummyCollectionData;
import org.nakedobjects.nof.reflect.remote.data.DummyNullValue;
import org.nakedobjects.nof.reflect.remote.data.DummyObjectData;
import org.nakedobjects.nof.reflect.remote.data.DummyReferenceData;
import org.nakedobjects.nof.reflect.remote.data.DummyValueData;
import org.nakedobjects.nof.reflect.remote.data.ObjectData;
import org.nakedobjects.nof.reflect.remote.data.ReferenceData;
import org.nakedobjects.nof.reflect.remote.spec.DummyField;
import org.nakedobjects.testing.DummyNakedCollection;
import org.nakedobjects.testing.DummyNakedObject;
import org.nakedobjects.testing.DummyNakedValue;
import org.nakedobjects.testing.DummyOid;
import org.nakedobjects.testing.DummyVersion;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;



public class ObjectEncoderMadePersistentTest extends TestCase {

    private TestSystem system;
    private Marshaller encoder;
    private DummyNakedObject rootObject;
    private DummyObjectData rootData;
    private DummyOid rootObjectsOid;

    protected void setUp() throws Exception {
        LogManager.getRootLogger().setLevel(Level.OFF);
        // system
        system = new TestSystem();
        system.init();
      //  encoder = new ObjectEncoderImpl();
        encoder = new Marshaller();
      //  encoder.setDataStructure(new DataStructure());
        encoder.setDataFactory(new DummyObjectDataFactory());

        /*
         * The following creates: 1) a data graph 2) a specification graph 3) an object graph. The object has
         * five field: three references (one which is null); one value; and one collection (referencing two
         * other objects)
         */
        // data graph
        rootData = new DummyObjectData(null, "root", false, null);
        rootData.setFieldContent(new Data[0]);

        DummyObjectData field1Data = new DummyObjectData(null, "ref-1", false, null);
        DummyObjectData field2Data = new DummyObjectData(null, "ref-2", false, null);
        DummyValueData field3Data = new DummyValueData("data", "string");
        DummyObjectData item1Data = new DummyObjectData(null, "item-1", false, null);
        DummyObjectData item2Data = new DummyObjectData(null, "item-2", false, null);
        DummyCollectionData field4Data = new DummyCollectionData(null, "collection",
                "ref-1", new ReferenceData[] { item1Data, item2Data }, null);
        DummyNullValue field5Data = new DummyNullValue("null");

        rootData.setFieldContent(new Data[] { field1Data, field2Data, field3Data, field4Data, field5Data });

        // specifications
        TestSpecification referencedObjectSpecification = new TestSpecification("reference field");
        referencedObjectSpecification.setupIsObject();
        NakedObjectField field1Spec = new DummyField("field-1", referencedObjectSpecification);
        NakedObjectField field2Spec = new DummyField("field-2", referencedObjectSpecification);

        TestSpecification valueSpecification = new TestSpecification("value field");
        valueSpecification.setupIsValue();
        NakedObjectField field3Spec = new DummyField("field-3", valueSpecification);

        TestSpecification collectionSpecification = new TestSpecification("collection field");
        collectionSpecification.setupIsCollection();
        NakedObjectField field4Spec = new DummyField("field-4", collectionSpecification);

        NakedObjectField field5Spec = new DummyField("field-5", referencedObjectSpecification);

        NakedObjectField[] rootObjectsFields = new NakedObjectField[] { field1Spec, field2Spec, field3Spec, field4Spec,
                field5Spec };
        TestSpecification rootObjectSpecification = new TestSpecification("root object");
        rootObjectSpecification.setupFields(rootObjectsFields);

        // object graph
        rootObject = new DummyNakedObject();
        rootObject.setupSpecification(rootObjectSpecification);
        rootObjectsOid = new DummyOid(10, true);
        rootObjectsOid.previous = new DummyOid(6);
        rootObject.setupOid(rootObjectsOid);
        rootObject.setupVersion(new DummyVersion(13));

        DummyNakedObject field1Object = new DummyNakedObject();
        // field1Object.setupSpecification(referencedObjectSpecification);
        field1Object.setupOid(new DummyOid(11));

        DummyNakedObject field2Object = new DummyNakedObject();
        // field2Object.setupSpecification(referencedObjectSpecification);
        DummyOid field2ObjectsOid = new DummyOid(12, true);
        field2ObjectsOid.previous = new DummyOid(10);
        field2Object.setupOid(field2ObjectsOid);

        DummyNakedValue field3Value = new DummyNakedValue();

        DummyNakedCollection field4Collection = new DummyNakedCollection();

        DummyNakedObject element1 = new DummyNakedObject();
        element1.setupOid(new DummyOid(21));

        DummyNakedObject element2 = new DummyNakedObject();
        DummyOid element2ObjectsOid = new DummyOid(22, true);
        element2ObjectsOid.previous = new DummyOid(20);
        element2.setupOid(element2ObjectsOid);

        field4Collection.setupElement(element1);
        field4Collection.setupElement(element2);

        DummyNakedObject field5Null = new DummyNakedObject();

        rootObject.setupFieldValue("field-1", field1Object);
        rootObject.setupFieldValue("field-2", field2Object);
        rootObject.setupFieldValue("field-3", field3Value);
        rootObject.setupFieldValue("field-4", field4Collection);
        rootObject.setupFieldValue("field-5", field5Null);
    }

    public void testPassesBackNullForNotPersistedRootObject() {
        rootObjectsOid.previous = null;

        ObjectData result = encoder.createMadePersistentGraph(rootData, rootObject);

        assertEquals(null, result);
    }

    public void testPassesBackRootObjectsOidWhenNewlyPersisted() {
        ObjectData result = encoder.createMadePersistentGraph(rootData, rootObject);

        assertNotNull(result);
        assertEquals(new DummyOid(10, true), result.getOid());
        assertEquals(new DummyOid(6), result.getOid().getPrevious());
        assertEquals(new DummyVersion(13), result.getVersion());
    }

    public void testReferencedObjectThatIsNotPersisted() {
        ObjectData result = encoder.createMadePersistentGraph(rootData, rootObject);

        assertEquals(null, result.getFieldContent()[0]);
    }

    public void testReferencedObjectThatIsPersisted() {
        ObjectData result = encoder.createMadePersistentGraph(rootData, rootObject);

        assertEquals(new DummyOid(12, true), ((ObjectData) result.getFieldContent()[1]).getOid());
        assertEquals(new DummyOid(10), ((ObjectData) result.getFieldContent()[1]).getOid().getPrevious());
    }

    public void testValueIsIgnored() {
        ObjectData result = encoder.createMadePersistentGraph(rootData, rootObject);

        assertEquals(null, result.getFieldContent()[2]);
    }

    public void testPersistentRefernceIgnored() {
        // note - this test where we only have a reference and not the object
        DummyReferenceData reference = new DummyReferenceData(new DummyOid(31), "type", new DummyVersion(2));
        rootData.setFieldContent(1, reference);

        ObjectData result = encoder.createMadePersistentGraph(rootData, rootObject);

        assertEquals(null, result.getFieldContent()[1]);
    }

    public void testCollection() {
        ObjectData result = encoder.createMadePersistentGraph(rootData, rootObject);

        CollectionData collectionData = (CollectionData) result.getFieldContent()[3];
        assertNotNull(collectionData);
        ReferenceData[] elements = collectionData.getElements();
        assertNotNull(elements);
        assertEquals(2, elements.length);

        assertNull(elements[0]);

        ObjectData element = (ObjectData) elements[1];
        assertEquals(new DummyOid(22, true), element.getOid());
        assertEquals(new DummyOid(20), element.getOid().getPrevious());
    }

    public void testRecursiveReference() {
        rootObject.setupFieldValue("field-1", rootObject);

        ObjectData result = encoder.createMadePersistentGraph(rootData, rootObject);

        assertEquals(new DummyOid(10, true), ((ObjectData) result.getFieldContent()[0]).getOid());
        assertEquals(new DummyOid(6), ((ObjectData) result.getFieldContent()[0]).getOid().getPrevious());
    }

    public void testNullReference() {
        ObjectData result = encoder.createMadePersistentGraph(rootData, rootObject);

        assertEquals(null, result.getFieldContent()[4]);
    }
}
// Copyright (c) Naked Objects Group Ltd.
