package org.nakedobjects.nos.remote.command.marshal;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.noa.adapter.Version;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.adapter.map.ObjectLoaderImpl;
import org.nakedobjects.nof.core.context.StaticContext;
import org.nakedobjects.nof.reflect.remote.data.ClientActionResultData;
import org.nakedobjects.nof.reflect.remote.data.CollectionData;
import org.nakedobjects.nof.reflect.remote.data.Data;
import org.nakedobjects.nof.reflect.remote.data.IdentityData;
import org.nakedobjects.nof.reflect.remote.data.KnownObjects;
import org.nakedobjects.nof.reflect.remote.data.NullData;
import org.nakedobjects.nof.reflect.remote.data.ObjectData;
import org.nakedobjects.nof.reflect.remote.data.ServerActionResultData;
import org.nakedobjects.nof.reflect.remote.data.ValueData;
import org.nakedobjects.testing.DummyNakedCollection;
import org.nakedobjects.testing.DummyNakedObject;
import org.nakedobjects.testing.DummyNakedValue;
import org.nakedobjects.testing.DummyOid;
import org.nakedobjects.testing.TestPersistor;
import org.nakedobjects.testing.TestSpecification;

import test.org.nakedobjects.objects.bom.Person;

public class ObjectEncoderTest2Disabled extends TestCase {

    private static final boolean NOT_RESOLVED = false;
    private static final boolean RESOLVED = true;

    
    Marshaller encoder = new Marshaller();
    StaticContext context;
    
    protected void setUp() throws Exception {
        LogManager.getRootLogger().setLevel(Level.OFF);
        
        context = (StaticContext) StaticContext.createInstance();
        context.setObjectLoader(new ObjectLoaderImpl() {
            public void initDomainObject(Object domainObject) {}});
        context.setObjectPersistor(new TestPersistor());
        
//        context.init();
        

        encoder.setDataFactory(new DummyObjectDataFactory());

    }
    
    public void testCreateServerActionResultWithNull() {
        ServerActionResultData data = encoder.createServerActionResult(null, null, new ObjectData[0], null, null, new String[0], new String[0]);
        assertNullData(data.getReturn());
    }

    public void testCreateServerActionResultWithCollection() {
        DummyNakedCollection collection = new DummyNakedCollection();
        TestSpecification type = new TestSpecification();
        collection.setupSpecification(type);

        ServerActionResultData data = encoder
                .createServerActionResult(collection, null, new ObjectData[0], null, null, new String[0], new String[0]);
        assertTrue(data.getReturn() instanceof CollectionData);
        assertEquals(type.getFullName(), data.getReturn().getType());
    }

    public void testCreateServerActionResultWithObject() {
        DummyNakedObject object = new DummyNakedObject();
        object.setupResolveState(ResolveState.PART_RESOLVED);
        TestSpecification type = new TestSpecification();
        object.setupSpecification(type);

        ServerActionResultData data = encoder.createServerActionResult(object, null, new ObjectData[0], null, null, new String[0], new String[0]);

        assertObjectData(object, NOT_RESOLVED, data.getReturn());

        assertTrue(data.getReturn() instanceof ObjectData);
        assertEquals(type.getFullName(), data.getReturn().getType());
    }

    public void testCreateClientActionResultWithObject() {
        ClientActionResultData data = encoder.createClientActionResult(new ObjectData[] {}, new Version[] {}, new ObjectData[0]);

        assertEquals(0, data.getChanged().length);
        assertEquals(0, data.getPersisted().length);
    }

    private void assertHasFields(final int noOfFields, final ObjectData rootData) {
        assertEquals(noOfFields, rootData.getFieldContent().length);
    }

    private void assertNullData(final Data ref) {
        assertTrue("instance of NullData", ref instanceof NullData);
    }

    private void assertReferenceData(final NakedObject rootObject, final Data od) {
        IdentityData data = (IdentityData) od;
        assertFalse("must be data for a reference, not an object", od instanceof ObjectData);
        assertEquals(rootObject.getOid(), data.getOid());
        assertEquals(rootObject.getVersion(), data.getVersion());
        assertEquals(rootObject.getSpecification().getFullName(), data.getType());

    }
    
    private void assertObjectData(final DummyNakedObject rootObject2, final boolean b, final Data data) {
        assertTrue("must be data for an object", data instanceof ObjectData);
        ObjectData objectData = (ObjectData) data;
        assertEquals(rootObject2.getOid(), objectData.getOid());
        assertEquals(rootObject2.getSpecification().getFullName(), objectData.getType());
        assertEquals(rootObject2.getVersion(), objectData.getVersion());
    }

    private void assertValueData(final DummyNakedValue value, final Data data) {
        ValueData valueData = (ValueData) data;
        assertEquals(value.getSpecification().getFullName(), valueData.getType());
        assertEquals(value.getObject(), valueData.getEncodedValue());
    }
    

    public void testPersistentObjectParameter() {
        NakedObject rootObject = context.getObjectLoader().createAdapterForTransient(new Person(), false);
        
        assertEquals(ResolveState.RESOLVED, rootObject.getResolveState());

        DummyOid rootOid = new DummyOid(1);
    //    rootObject.setupOid(rootOid);

        Data[] results = encoder.createParameters(new NakedObjectSpecification[] { new TestSpecification(),
                new TestSpecification() }, new Naked[] { rootObject, null }, new KnownObjects());

        assertEquals(2, results.length);
        assertReferenceData(rootObject, results[0]);
        assertNullData(results[1]);
    }
/*
    public void testResolvedCreatesObjectData() {
        rootObject.setupResolveState(ResolveState.RESOLVED);
        rootObject.setupOid(new DummyOid(2));

        Data od = encoder.createActionTarget(rootObject, new KnownObjects());
        assertReferenceData(rootObject, od);
    }


    public void testGhostsCreateReferencetData() {
        rootObject.setupResolveState(ResolveState.GHOST);
        rootObject.setupOid(new DummyOid(2));

        ReferenceData rd = encoder.createActionTarget(rootObject, new KnownObjects());
        assertEquals(ResolveState.GHOST, rootObject.getResolveState());
        assertEquals(new DummyOid(2), rd.getOid());
        assertEquals(DummyReferenceData.class, rd.getClass());
    }
    

    public void testForResovleField() {
        field1.setupOid(new DummyOid(7));
        field1_1.setupOid(new DummyOid(11));
        rootObject.setupOid(new DummyOid(13));

        encoder.createForResolveField(rootObject, "field1");
    }
    */
    }


// Copyright (c) Naked Objects Group Ltd.
