package org.nakedobjects.nos.remote.command.marshal;

import java.util.Date;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.noa.adapter.Version;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.adapter.value.IntAdapter;
import org.nakedobjects.nof.reflect.remote.data.ClientActionResultData;
import org.nakedobjects.nof.reflect.remote.data.CollectionData;
import org.nakedobjects.nof.reflect.remote.data.Data;
import org.nakedobjects.nof.reflect.remote.data.DummyReferenceData;
import org.nakedobjects.nof.reflect.remote.data.DummyValueData;
import org.nakedobjects.nof.reflect.remote.data.IdentityData;
import org.nakedobjects.nof.reflect.remote.data.KnownObjects;
import org.nakedobjects.nof.reflect.remote.data.NullData;
import org.nakedobjects.nof.reflect.remote.data.ObjectData;
import org.nakedobjects.nof.reflect.remote.data.ReferenceData;
import org.nakedobjects.nof.reflect.remote.data.ServerActionResultData;
import org.nakedobjects.nof.reflect.remote.data.ValueData;
import org.nakedobjects.nof.reflect.remote.spec.DummyField;
import org.nakedobjects.testing.DummyNakedCollection;
import org.nakedobjects.testing.DummyNakedObject;
import org.nakedobjects.testing.DummyNakedValue;
import org.nakedobjects.testing.DummyOid;
import org.nakedobjects.testing.DummyVersion;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;



public class ObjectEncoderTestDisabled extends TestCase {

    private static final boolean NOT_RESOLVED = false;
    private static final boolean RESOLVED = true;

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(ObjectEncoderTestDisabled.class);
    }

    private TestSpecification field4Specification;
    private Marshaller encoder;
    private DummyNakedObject field1;
    private DummyNakedObject field1_1;
    private TestSpecification field1_1Specification;
    private TestSpecification field1Specification;
    private DummyNakedObject rootObject;
    private TestSpecification rootSpecification;
    private TestSystem system;
    private Date value;
    private DummyNakedValue field2;
    private NakedObjectSpecification valueSpecification;
    private TestSpecification field3Specification;
    private DummyNakedCollection field3;
    private DummyNakedObject element1;
    private DummyNakedObject element2;

    /**
     * Sets up an object graph
     * 
     * rootObject - v11 | +--- referencedObjectField - v12 | | | +-- referencedReferencedObjectField - v13 | | |
     * +-- root object (cyclical reference) | +--- valueField | +--- collection - v27 | | | +-- element1 - v17 | | | | |
     * +-- root object (cyclical reference) | | | +-- element2 - v19 | +--- null (empty)
     */
    protected void setUp() throws Exception {
        LogManager.getRootLogger().setLevel(Level.OFF);

        system = new TestSystem();
        system.init();

        encoder = new Marshaller();
     //   encoder = new ObjectEncoderImpl();
        //          encoder.setDataStructure(new DataStructure());
        encoder.setDataFactory(new DummyObjectDataFactory());

        // field 1 - reference to another object
        field1Specification = new TestSpecification();
        field1Specification.fields = new NakedObjectField[1];

        field1 = new DummyNakedObject();
        field1.setupVersion(new DummyVersion(12));
        field1.setupResolveState(ResolveState.RESOLVED);
        field1.setupSpecification(field1Specification);

        field1_1Specification = new TestSpecification();
        field1_1Specification.fields = new NakedObjectField[1];
        field1Specification.fields[0] = new DummyField("field5", field1Specification);

        field1_1 = new DummyNakedObject();
        field1_1.setupVersion(new DummyVersion(13));
        field1_1.setupResolveState(ResolveState.RESOLVED);
        field1_1.setupSpecification(field1_1Specification);
        field1.setupFieldValue("field5", field1_1);

        // value field
        field2 = new DummyNakedValue();
        value = new Date();
        field2.setupObject(value);
        valueSpecification = field2.getSpecification();

        // collection field - with two elements
        TestSpecification element1Specification = new TestSpecification();
        element1Specification.fields = new NakedObjectField[1];
        element1 = new DummyNakedObject();
        element1.setupVersion(new DummyVersion(17));
        element1.setupResolveState(ResolveState.RESOLVED);
        element1.setupSpecification(element1Specification);

        TestSpecification element2Specification = new TestSpecification();
        element2Specification.fields = new NakedObjectField[0];
        element2 = new DummyNakedObject();
        element2.setupVersion(new DummyVersion(19));
        element2.setupResolveState(ResolveState.RESOLVED);
        element2.setupSpecification(element2Specification);

        field3Specification = new TestSpecification();
        field3Specification.setupIsCollection();
        field3 = new DummyNakedCollection();
        field3.setupResolveState(ResolveState.RESOLVED);
        field3.setupSpecification(field3Specification);
        field3.setupVersion(new DummyVersion(27));
        field3.init(new NakedObject[] { element1, element2 });

        // empty reference field
        field4Specification = new TestSpecification();
        field4Specification.fields = new NakedObjectField[0];

        // root object
        rootSpecification = new TestSpecification();
        rootSpecification.fields = new NakedObjectField[4];
        rootSpecification.fields[0] = new DummyField("field1", field1Specification);
        rootSpecification.fields[1] = new DummyField("field2", valueSpecification);
        rootSpecification.fields[2] = new DummyField("field3", field3Specification);
        rootSpecification.fields[3] = new DummyField("field4", field4Specification);

        rootObject = new DummyNakedObject();
        rootObject.setupVersion(new DummyVersion(11));
        rootObject.setupResolveState(ResolveState.RESOLVED);
        rootObject.setupSpecification(rootSpecification);
        rootObject.setupFieldValue("field1", field1);
        rootObject.setupFieldValue("field2", field2);
        rootObject.setupFieldValue("field3", field3);

        // cyclic references
        DummyNakedObject field1_1_1 = rootObject;
        field1_1Specification.fields[0] = new DummyField("backref1", rootSpecification);
        field1_1.setupFieldValue("backref1", field1_1_1);

        DummyNakedObject element1_field1 = rootObject;
        element1Specification.fields[0] = new DummyField("backref2", rootSpecification);
        element1.setupFieldValue("backref2", element1_field1);
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }

    public void testCreateServerActionResultWithNull() {
        ServerActionResultData data = encoder.createServerActionResult(null, null, new ObjectData[0], null, null, new String[0], new String[0]);
        assertNullData(data.getReturn());
    }

    public void testCreateServerActionResultWithCollection() {
        DummyNakedCollection collection = new DummyNakedCollection();
        TestSpecification type = new TestSpecification();
        collection.setupSpecification(type);

        ServerActionResultData data = encoder
                .createServerActionResult(collection, null, new ObjectData[0], null, null, new String[0], new String[0]);
        assertTrue(data.getReturn() instanceof CollectionData);
        assertEquals(type.getFullName(), data.getReturn().getType());
    }

    public void testCreateServerActionResultWithObject() {
        DummyNakedObject object = new DummyNakedObject();
        object.setupResolveState(ResolveState.PART_RESOLVED);
        TestSpecification type = new TestSpecification();
        object.setupSpecification(type);

        ServerActionResultData data = encoder.createServerActionResult(object, null, new ObjectData[0], null, null, new String[0], new String[0]);

        assertObjectData(object, NOT_RESOLVED, data.getReturn());

        assertTrue(data.getReturn() instanceof ObjectData);
        assertEquals(type.getFullName(), data.getReturn().getType());
    }

    public void testCreateClientActionResultWithObject() {
        ClientActionResultData data = encoder.createClientActionResult(new ObjectData[] {}, new Version[] {}, new ObjectData[0]);

        assertEquals(0, data.getChanged().length);
        assertEquals(0, data.getPersisted().length);
    }

    public void testCreateCompletePersistentGraph() {
        DummyOid rootOid = new DummyOid(1);
        rootObject.setupOid(rootOid);
        DummyOid referencedOid = new DummyOid(3);
        field1.setupOid(referencedOid);
        DummyOid referencedOid2 = new DummyOid(5);
        field1_1.setupOid(referencedOid2);

        ObjectData rootData = (ObjectData) encoder.createCompletePersistentGraph(rootObject);
        assertObjectData(rootObject, RESOLVED, rootData);
        assertHasFields(4, rootData);

        ObjectData referencedFieldData = (ObjectData) rootData.getFieldContent()[0];
        assertObjectData(field1, RESOLVED, referencedFieldData);
        assertHasFields(1, referencedFieldData);

        ObjectData referencedFieldData2 = (ObjectData) referencedFieldData.getFieldContent()[0];
        assertObjectData(field1_1, RESOLVED, referencedFieldData2);
        assertHasFields(1, referencedFieldData2);

        assertValueData(field2, rootData.getFieldContent()[1]);
        assertNullData(rootData.getFieldContent()[3]);

        CollectionData collectionData = (CollectionData) rootData.getFieldContent()[2];
        assertEquals(2, collectionData.getElements().length);

        // check back references
        ReferenceData fieldOfField = (ReferenceData) referencedFieldData2.getFieldContent()[0];
        assertReferenceData(rootObject, fieldOfField);
        assertEquals(fieldOfField, rootData);

        ObjectData referenceData = (ObjectData) collectionData.getElements()[0];
        ReferenceData fieldOfElement = (ReferenceData) referenceData.getFieldContent()[0];
        assertReferenceData(rootObject, fieldOfElement);
    }

    public void testCreateCompletePersistentGraphWithGhosts() {
        rootObject.setupOid(new DummyOid(9));
        field1.setupOid(new DummyOid(11));
        field1.setupResolveState(ResolveState.GHOST);
        element1.setupOid(new DummyOid(13));
        element1.setupResolveState(ResolveState.GHOST);
        element2.setupOid(new DummyOid(15));
        element2.setupResolveState(ResolveState.GHOST);

        ObjectData rootData = (ObjectData) encoder.createCompletePersistentGraph(rootObject);

        assertObjectData(rootObject, NOT_RESOLVED, rootData);
        assertHasFields(4, rootData);

        assertReferenceData(field1, rootData.getFieldContent()[0]);
        assertValueData(field2, rootData.getFieldContent()[1]);
        assertNullData(rootData.getFieldContent()[3]);

        CollectionData collectionData = (CollectionData) rootData.getFieldContent()[2];
        assertReferenceData(element1, collectionData.getElements()[0]);
        assertReferenceData(element2, collectionData.getElements()[1]);

    }

    /**
     * For updates the whole object is passed across, but not any of its children.
     */
    public void testCreateForUpdate() {
        field1.setupOid(new DummyOid(3));
        element1.setupOid(new DummyOid(3));
        element2.setupOid(new DummyOid(3));

        ObjectData rootData = (ObjectData) encoder.createForUpdate(rootObject);
        assertEquals(ResolveState.RESOLVED, rootObject.getResolveState());

        assertObjectData(rootObject, RESOLVED, rootData);
        assertHasFields(4, rootData);

        assertReferenceData(field1, rootData.getFieldContent()[0]);
        assertValueData(field2, rootData.getFieldContent()[1]);
        assertNullData(rootData.getFieldContent()[3]);

        CollectionData collectionData = (CollectionData) rootData.getFieldContent()[2];
        assertReferenceData(element1, collectionData.getElements()[0]);
        assertReferenceData(element2, collectionData.getElements()[1]);
    }

    public void testCreateForChangedObject() {
        field1.setupOid(new DummyOid(3));
        element1.setupOid(new DummyOid(3));
        element2.setupOid(new DummyOid(3));

        ObjectData rootData = (ObjectData) encoder.createGraphForChangedObject(rootObject, new KnownObjects());
        assertEquals(ResolveState.RESOLVED, rootObject.getResolveState());

        assertObjectData(rootObject, RESOLVED, rootData);
        assertHasFields(4, rootData);

        assertReferenceData(field1, rootData.getFieldContent()[0]);
        assertValueData(field2, rootData.getFieldContent()[1]);
        assertNullData(rootData.getFieldContent()[3]);

        CollectionData collectionData = (CollectionData) rootData.getFieldContent()[2];
        assertReferenceData(element1, collectionData.getElements()[0]);
        assertReferenceData(element2, collectionData.getElements()[1]);
    }

    public void testCreateMakePersistentGraphWhereAllReferencesArePersistent() {
        rootObject.setupResolveState(ResolveState.TRANSIENT);
        field1.setupOid(new DummyOid(194));
        element1.setupOid(new DummyOid(198));
        element2.setupOid(new DummyOid(196));

        ObjectData rootData = (ObjectData) encoder.createMakePersistentGraph(rootObject, new KnownObjects());
        assertObjectData(rootObject, RESOLVED, rootData);

        assertReferenceData(field1, rootData.getFieldContent()[0]);
        assertValueData(field2, rootData.getFieldContent()[1]);
        assertNullData(rootData.getFieldContent()[3]);

        CollectionData collectionData = (CollectionData) rootData.getFieldContent()[2];
        assertReferenceData(element1, collectionData.getElements()[0]);
        assertReferenceData(element2, collectionData.getElements()[1]);
    }

    public void testCreateMakePersistentGraphWhereAllTransient() {
        rootObject.setupResolveState(ResolveState.TRANSIENT);
        field1.setupResolveState(ResolveState.TRANSIENT);
        field1_1.setupResolveState(ResolveState.TRANSIENT);
        element1.setupResolveState(ResolveState.TRANSIENT);
        element2.setupResolveState(ResolveState.TRANSIENT);

        ObjectData rootData = (ObjectData) encoder.createMakePersistentGraph(rootObject, new KnownObjects());
        assertObjectData(rootObject, RESOLVED, rootData);
        assertHasFields(4, rootData);

        ObjectData referencedFieldData = (ObjectData) rootData.getFieldContent()[0];
        assertObjectData(field1, RESOLVED, referencedFieldData);
        assertHasFields(1, referencedFieldData);

        ObjectData referencedFieldData2 = (ObjectData) referencedFieldData.getFieldContent()[0];
        assertObjectData(field1_1, RESOLVED, referencedFieldData2);
        assertHasFields(1, referencedFieldData2);

        assertValueData(field2, rootData.getFieldContent()[1]);

        assertNullData(rootData.getFieldContent()[3]);

        CollectionData collectionData = (CollectionData) rootData.getFieldContent()[2];
        assertEquals(2, collectionData.getElements().length);

        assertObjectData(element1, RESOLVED, collectionData.getElements()[0]);
        assertObjectData(element2, RESOLVED, collectionData.getElements()[1]);

        // check back references
        ObjectData fieldOfField = (ObjectData) referencedFieldData2.getFieldContent()[0];
        assertSame(fieldOfField, rootData);

        ObjectData referenceData = (ObjectData) collectionData.getElements()[0];
        ObjectData fieldOfElement = (ObjectData) referenceData.getFieldContent()[0];
        assertSame(fieldOfElement, rootData);
    }

    public void testPersistentObjectParameter() {
        assertEquals(ResolveState.RESOLVED, rootObject.getResolveState());

        DummyOid rootOid = new DummyOid(1);
        rootObject.setupOid(rootOid);

        Data[] results = encoder.createParameters(new NakedObjectSpecification[] { new TestSpecification(),
                new TestSpecification() }, new Naked[] { rootObject, null }, new KnownObjects());

        assertEquals(2, results.length);
        assertReferenceData(rootObject, results[0]);
        assertNullData(results[1]);
    }

    public void testResolvedCreatesObjectData() {
        rootObject.setupResolveState(ResolveState.RESOLVED);
        rootObject.setupOid(new DummyOid(2));

        Data od = encoder.createActionTarget(rootObject, new KnownObjects());
        assertReferenceData(rootObject, od);
    }

    private void assertReferenceData(final DummyNakedObject rootObject, final Data od) {
        IdentityData data = (IdentityData) od;
        assertFalse("must be data for a reference, not an object", od instanceof ObjectData);
        assertEquals(rootObject.getOid(), data.getOid());
        assertEquals(rootObject.getVersion(), data.getVersion());
        assertEquals(rootObject.getSpecification().getFullName(), data.getType());

    }

    public void testGhostsCreateReferencetData() {
        rootObject.setupResolveState(ResolveState.GHOST);
        rootObject.setupOid(new DummyOid(2));

        ReferenceData rd = encoder.createActionTarget(rootObject, new KnownObjects());
        assertEquals(ResolveState.GHOST, rootObject.getResolveState());
        assertEquals(new DummyOid(2), rd.getOid());
        assertEquals(DummyReferenceData.class, rd.getClass());
    }

    public void testTransientObjectParameter() {
        rootObject.setupResolveState(ResolveState.TRANSIENT);
        field1.setupResolveState(ResolveState.TRANSIENT);
        assertEquals(ResolveState.TRANSIENT, rootObject.getResolveState());

        field1_1.setupOid(new DummyOid(123));
        element1.setupOid(new DummyOid(125));
        element2.setupOid(new DummyOid(127));

        Data[] data = encoder.createParameters(new NakedObjectSpecification[] { new TestSpecification() },
                new Naked[] { rootObject }, new KnownObjects());

        ObjectData rootData = (ObjectData) data[0];
        assertObjectData(rootObject, RESOLVED, rootData);
        assertHasFields(4, rootData);

        ObjectData referencedFieldData = (ObjectData) rootData.getFieldContent()[0];
        assertObjectData(field1, RESOLVED, referencedFieldData);
        assertHasFields(1, referencedFieldData);

        assertReferenceData(field1_1, referencedFieldData.getFieldContent()[0]);

        assertValueData(field2, rootData.getFieldContent()[1]);

        NullData emptyFieldData = (NullData) rootData.getFieldContent()[3];
        assertNullData(emptyFieldData);
    }

    private void assertNullData(final Data ref) {
        assertTrue("instance of NullData", ref instanceof NullData);
    }

    private void assertValueData(final DummyNakedValue value, final Data data) {
        ValueData valueData = (ValueData) data;
        assertEquals(value.getSpecification().getFullName(), valueData.getType());
        assertEquals(value.getObject(), valueData.getEncodedValue());

    }

    private void assertHasFields(final int noOfFields, final ObjectData rootData) {
        assertEquals(noOfFields, rootData.getFieldContent().length);
    }

    private void assertObjectData(final DummyNakedObject rootObject2, final boolean b, final Data data) {
        assertTrue("must be data for an object", data instanceof ObjectData);
        ObjectData objectData = (ObjectData) data;
        assertEquals(rootObject2.getOid(), objectData.getOid());
        assertEquals(rootObject2.getSpecification().getFullName(), objectData.getType());
        assertEquals(rootObject2.getVersion(), objectData.getVersion());
    }

    public void testValueParameter() {
        DummyNakedValue dummyNakedValue = new DummyNakedValue();
        dummyNakedValue.setupObject(new Integer(123));

        Data[] data = (Data[]) encoder.createParameters(new NakedObjectSpecification[] { new TestSpecification() },
                new Naked[] { dummyNakedValue }, new KnownObjects());

        DummyValueData expected = new DummyValueData(new IntAdapter(new Integer(123)).asEncodedString(), dummyNakedValue.getSpecification().getFullName());
        assertEquals(expected, data[0]);
    }

    public void testForResovleField() {
        field1.setupOid(new DummyOid(7));
        field1_1.setupOid(new DummyOid(11));
        rootObject.setupOid(new DummyOid(13));

        encoder.createForResolveField(rootObject, "field1");
    }
}
// Copyright (c) Naked Objects Group Ltd.
